
#include "Arduino.h"
#define SET_ACQUISITION 100
#define STOP_ACQUISITION 101
#define NBUF 0x2
#define NBUF_MASK 0x1 // (NBUF-1)
#define BUF_SIZE 0x100
#define BUF_MASK 0xFF // (BUF_SIZE-1)
#define MAX_NVOIES 4
volatile uint16_t buf[MAX_NVOIES][NBUF][BUF_SIZE];
volatile uint8_t compteur_buf,compteur_buf_transmis;
volatile uint16_t indice_buf;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
uint32_t nblocs;
uint8_t nvoies;
uint8_t multiplex[MAX_NVOIES];
uint8_t sans_fin;
uint8_t flag;
            
void timer1_init(uint32_t period) {
    TCCR1A = 0;
    TCCR1B = 0;
    TCCR1B |= (1 << WGM12); // mode CTC avec OCR1A pour le maximum
    uint32_t top = (F_CPU/1000000*period);
    int clock = 1;
    while ((top>0xFFFF)&&(clock<5)) {
          clock++;
          top = (F_CPU/1000000*period/diviseur[clock]);
      }
    OCR1A = top; // période
    TIMSK1 = (1 << OCIE1A); // interruption lorsque TCNT1=OCR1A
    TCCR1B |= clock;
}
               
void adc_init(uint8_t prescaler) {
  ADCSRA = 0;
  ADCSRA |= (1 << ADEN); // enable ADC
  ADCSRA |= prescaler ;
  ADCSRB = 0;
}
               
ISR(TIMER1_COMPA_vect) {
    uint8_t v;
    for (v=0; v<nvoies; v++) {
       ADMUX  = 0b01000000 | multiplex[v]; // REFS0 et multiplex
       //ADCSRB |= (multiplex & 0b00100000) >> 2; // pour multiplex sur 6 bits
       ADCSRA |= 0b01000000; // start ADC
       while (ADCSRA & 0b01000000);
       buf[v][compteur_buf][indice_buf] = ADCL | (ADCH << 8);
    }
    indice_buf++;
    if (indice_buf == BUF_SIZE) {
       indice_buf = 0;
       compteur_buf = (compteur_buf+1)&NBUF_MASK;
     }   
}
               
void stop_acquisition() {
   TCCR1B &= 0b11111000; 
}
               
void setup() {
  char c;
  Serial.begin(500000);
  Serial.setTimeout(0);
  c = 0;
  Serial.write(c);
  c = 255;
  Serial.write(c);
  c = 0;
  Serial.write(c);
  compteur_buf = compteur_buf_transmis = 0;
  indice_buf = 0;
  flag = 0;
}
               
void lecture_acquisition() {
  uint32_t c1,c2,c3,c4;
  uint8_t mult,prescaler;
  uint32_t period;
  uint8_t v;
  while (Serial.available()<2) {};
  nvoies = Serial.read();
  prescaler = Serial.read();
  while (Serial.available()<nvoies) {};
  for (v=0; v<nvoies; v++) {
      mult = Serial.read();
      if (v < MAX_NVOIES) multiplex[v] = mult;
  }
  if (nvoies > MAX_NVOIES) nvoies = MAX_NVOIES;
  while (Serial.available()<4) {};
  c1 = Serial.read();
  c2 = Serial.read();
  c3 = Serial.read();
  c4 = Serial.read();
  period = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
  while (Serial.available()<4) {};
  c1 = Serial.read();
  c2 = Serial.read();
  c3 = Serial.read();
  c4 = Serial.read();
  nblocs = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
  if (nblocs==0) {
        sans_fin = 1;
        nblocs = 1;
   }
   else sans_fin = 0;
   compteur_buf=compteur_buf_transmis=0;
   indice_buf = 0;
  cli();
  timer1_init(period);
  adc_init(prescaler);
  sei();
}
               
void lecture_serie() {
   char com;
   if (Serial.available()>0) {
        com = Serial.read();
        if (com==SET_ACQUISITION) lecture_acquisition();
        if (com==STOP_ACQUISITION) stop_acquisition();
   }
}
               
void loop() {
  uint8_t v;
  if ((compteur_buf_transmis!=compteur_buf)&&(nblocs>0)) {
      for (v=0; v<nvoies; v++) Serial.write((uint8_t *)buf[v][compteur_buf_transmis],BUF_SIZE*2);
      compteur_buf_transmis=(compteur_buf_transmis+1)&NBUF_MASK;
      if (sans_fin==0) {
            nblocs--;
            if (nblocs==0) stop_acquisition();
      }
  }
  else lecture_serie();
}
               